"use strict";
var common_vendor = require("../../../../../common/vendor.js");
var pages_TUIKit_TUICore_server_IComponentServer = require("../IComponentServer.js");
var store_index = require("../../../../../store/index.js");
class TUIConversationServer extends pages_TUIKit_TUICore_server_IComponentServer.IComponentServer {
  constructor(TUICore) {
    super();
    this.store = store_index.store.state.timStore;
    this.TUICore = common_vendor.index.$TUIKit;
    common_vendor.index.setStorageSync(`TIM_${common_vendor.index.$chat_SDKAppID}_isTUIKit`, true);
    if (common_vendor.index.$aegis) {
      common_vendor.index.$aegis.reportEvent({
        name: "platform",
        ext1: "platform-MP-WEIXIN",
        ext2: "uniTuikitExternalVue3",
        ext3: `${common_vendor.index.$chat_SDKAppID}`
      });
    }
    this.bindTIMEvent();
    if (common_vendor.index.$chat_isSDKReady) {
      this.getConversationList();
    }
  }
  bindTIMEvent() {
    this.TUICore.on(common_vendor.index.$TIM.EVENT.CONVERSATION_LIST_UPDATED, this.handleConversationListUpdate, this);
  }
  unbindTIMEvent() {
    this.TUICore.off(common_vendor.index.$TIM.EVENT.CONVERSATION_LIST_UPDATED, this.handleConversationListUpdate);
  }
  handleConversationListUpdate(res) {
    common_vendor.index.hideLoading();
    if (res.data.length === 0)
      ;
    common_vendor.index.$emit("refreshMsgCount");
    res.data.forEach(function(element) {
      console.log("\u6536\u5230\u4F1A\u8BDD\u6D88\u606F");
      console.log(element);
    });
    store_index.store.commit("timStore/setConversationList", res.data);
  }
  destroyed() {
    this.unbindTIMEvent();
  }
  async getConversationList() {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await common_vendor.index.$TUIKit.getConversationList();
        store_index.store.commit("timStore/setConversationList", imResponse.data.conversationList);
        common_vendor.index.hideLoading();
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
  async getConversationProfile(conversationID) {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await this.TUICore.getConversationProfile(conversationID);
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
  async setConversationValue(conversationID, data) {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await this.TUICore.setConversationCustomData({
          conversationIDList: [conversationID],
          customData: data
        });
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
  async deleteConversation(conversationID) {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await this.TUICore.deleteConversation(conversationID);
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
  async pinConversation(options) {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await this.TUICore.pinConversation(options);
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
  async muteConversation(options) {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await this.TUICore.setMessageRemindType(options);
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
  async setMessageRead(conversationID) {
    return new Promise(async (resolve, reject) => {
      try {
        const imResponse = await this.TUICore.setMessageRead({
          conversationID
        });
        resolve(imResponse);
      } catch (error) {
        reject(error);
      }
    });
  }
}
exports.TUIConversationServer = TUIConversationServer;
